@@ -82,7 +82,7 @@ extern void preprocessor_cc_free_level(ConditionalConfig **cc_list, int level);
 extern void preprocessor_cc_free_list(ConditionalConfig *cc);
 extern void preprocessor_resolve_conditionals_ce(ConfigEntry **ce_list, PreprocessorPhase phase);
 extern void preprocessor_resolve_conditionals_all(PreprocessorPhase phase);
-extern void free_config_defines(void);
+extern void init_config_defines(void);
 extern void preprocessor_replace_defines(char **item, ConfigEntry *ce);
 
 /*
@@ -100,7 +100,6 @@ extern MODVAR ConfigItem_tld		*conf_tld;
 extern MODVAR ConfigItem_oper		*conf_oper;
 extern MODVAR ConfigItem_listen	*conf_listen;
 extern MODVAR ConfigItem_allow		*conf_allow;
-extern MODVAR ConfigItem_vhost		*conf_vhost;
 extern MODVAR ConfigItem_link		*conf_link;
 extern MODVAR ConfigItem_sni		*conf_sni;
 extern MODVAR ConfigItem_ban		*conf_ban;
@@ -114,7 +113,6 @@ extern MODVAR ConfigItem_proxy		*conf_proxy;
 extern void		completed_connection(int, int, void *);
 extern void clear_unknown();
 extern EVENT(e_unload_module_delayed);
-extern EVENT(throttling_check_expire);
 
 extern void  module_loadall(void);
 extern long set_usermode(const char *umode);
@@ -133,6 +131,7 @@ extern void config_error_unknownopt(const char *filename, int line, const char *
 extern void config_error_noname(const char *filename, int line, const char *block);
 extern void config_error_blank(const char *filename, int line, const char *block);
 extern void config_error_empty(const char *filename, int line, const char *block, const char *entry);
+extern int config_detect_duplicate(int *var, ConfigEntry *ce, int *errors);
 extern void config_warn_duplicate(const char *filename, int line, const char *entry);
 extern int config_is_blankorempty(ConfigEntry *cep, const char *block);
 extern MODVAR int config_verbose;
@@ -151,13 +150,18 @@ extern ConfigItem_tld		*find_tld(Client *cptr);
 extern ConfigItem_link		*find_link(const char *servername);
 extern ConfigItem_ban 		*find_ban(Client *, const char *host, short type);
 extern ConfigItem_ban 		*find_banEx(Client *,const char *host, short type, short type2);
-extern ConfigItem_vhost	*find_vhost(const char *name);
 extern ConfigItem_deny_channel *find_channel_allowed(Client *cptr, const char *name);
 extern ConfigItem_alias	*find_alias(const char *name);
 extern ConfigItem_help 	*find_Help(const char *command);
 
 extern OperPermission ValidatePermissionsForPath(const char *path, Client *client, Client *victim, Channel *channel, const void *extra);
 extern void OperClassValidatorDel(OperClassValidator *validator);
+extern OperClassACLPath *OperClass_parsePath(const char *path);
+extern OperClassACL *OperClass_FindACL(OperClassACL *acl, char *name);
+extern OperPermission ValidatePermissionsForPathEx(OperClassACL *acl, OperClassACLPath *path, OperClassCheckParams *params);
+extern void OperClass_freePath(OperClassACLPath *path);
+extern OperClassACL *_conf_parseACL(const char *name, ConfigEntry *ce);
+extern void free_operclass_struct(OperClass *o);
 
 extern ConfigItem_ban  *find_ban_ip(Client *client);
 extern void add_ListItem(ListStruct *, ListStruct **);
@@ -377,9 +381,6 @@ extern const char *pretty_date(time_t t);
 extern time_t server_time_to_unix_time(const char *tbuf);
 extern time_t rfc2616_time_to_unix_time(const char *tbuf);
 extern const char *rfc2616_time(time_t clock);
-extern void exit_client(Client *client, MessageTag *recv_mtags, const char *comment);
-extern void exit_client_fmt(Client *client, MessageTag *recv_mtags, FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf, 3, 4)));
-extern void exit_client_ex(Client *client, Client *origin, MessageTag *recv_mtags, const char *comment);
 extern void initstats();
 extern const char *check_string(const char *);
 extern char *make_nick_user_host(const char *, const char *, const char *);
@@ -431,8 +432,6 @@ extern void del_queries(const char *);
 #define NICK_HASH_TABLE_SIZE 32768
 #define CHAN_HASH_TABLE_SIZE 32768
 #define WHOWAS_HASH_TABLE_SIZE 32768
-#define THROTTLING_HASH_TABLE_SIZE 8192
-#define IPUSERS_HASH_TABLE_SIZE 8192
 extern uint64_t siphash(const char *in, const char *k);
 extern uint64_t siphash_raw(const char *in, size_t len, const char *k);
 extern uint64_t siphash_nocase(const char *in, const char *k);
@@ -458,13 +457,6 @@ extern Client *hash_find_id(const char *, Client *);
 extern Client *hash_find_nickatserver(const char *, Client *);
 extern Channel *find_channel(const char *name);
 extern Client *hash_find_server(const char *, Client *);
-extern IpUsersBucket *find_ipusers_bucket(Client *client);
-extern IpUsersBucket *add_ipusers_bucket(Client *client);
-extern void decrease_ipusers_bucket(Client *client);
-extern MODVAR struct ThrottlingBucket *ThrottlingHash[THROTTLING_HASH_TABLE_SIZE];
-extern MODVAR IpUsersBucket *IpUsersHash_ipv4[IPUSERS_HASH_TABLE_SIZE];
-extern MODVAR IpUsersBucket *IpUsersHash_ipv6[IPUSERS_HASH_TABLE_SIZE];
-
 
 /* Mode externs
 */
@@ -705,6 +697,7 @@ extern void verify_opercount(Client *, const char *);
 extern int valid_host(const char *host, int strict);
 extern int valid_username(const char *username);
 extern int valid_vhost(const char *userhost);
+extern int potentially_valid_vhost(const char *userhost);
 extern int count_oper_sessions(const char *);
 extern char *unreal_mktemp(const char *dir, const char *suffix);
 extern char *unreal_getpathname(const char *filepath, char *path);
@@ -717,6 +710,7 @@ extern int unreal_copyfile(const char *src, const char *dest);
 extern int unreal_copyfileex(const char *src, const char *dest, int tryhardlink);
 extern time_t unreal_getfilemodtime(const char *filename);
 extern void unreal_setfilemodtime(const char *filename, time_t mtime);
+extern int unreal_touch(const char *filename, time_t mtime);
 extern void DeleteTempModules(void);
 extern MODVAR Extban *extbaninfo;
 extern Extban *findmod_by_bantype(const char *str, const char **remainder);
@@ -745,6 +739,7 @@ extern int has_actions_of_type(BanAction *actions, BanActionValue what);
 extern int only_soft_actions(BanAction *actions);
 extern const char *ban_actions_to_string(BanAction *actions);
 extern void lower_ban_action_to_maximum(BanAction *actions, BanActionValue limit_action);
+extern SpamfilterShowMessageContentOnHit spamfilter_show_message_content_on_hit_strtoval(const char *s);
 extern int spamfilter_gettargets(const char *s, Client *client);
 extern char *spamfilter_target_inttostring(int v);
 extern char *our_strcasestr(const char *haystack, const char *needle);
@@ -800,7 +795,7 @@ extern MODVAR MultiLineMode *(*set_mode)(Channel *channel, Client *cptr, int par
                             char pvar[MAXMODEPARAMS][MODEBUFLEN + 3]);
 extern MODVAR void (*set_channel_mode)(Channel *channel, MessageTag *mtags, const char *modes, const char *parameters);
 extern MODVAR void (*set_channel_topic)(Client *client, Channel *channel, MessageTag *recv_mtags, const char *topic, const char *set_by, time_t set_at);
-extern MODVAR void (*cmd_umode)(Client *, MessageTag *, int, const char **);
+extern MODVAR void (*cmd_umode)(ClientContext *, Client *, MessageTag *, int, const char **);
 extern MODVAR int (*register_user)(Client *client);
 extern MODVAR int (*tkl_hash)(unsigned int c);
 extern MODVAR char (*tkl_typetochar)(int type);
@@ -808,8 +803,9 @@ extern MODVAR int (*tkl_chartotype)(char c);
 extern MODVAR char (*tkl_configtypetochar)(const char *name);
 extern MODVAR const char *(*tkl_type_string)(TKL *tk);
 extern MODVAR const char *(*tkl_type_config_string)(TKL *tk);
-extern MODVAR TKL *(*tkl_add_serverban)(int type, const char *usermask, const char *hostmask, const char *reason, const char *setby,
-                                            time_t expire_at, time_t set_at, int soft, int flags);
+extern MODVAR TKL *(*tkl_add_serverban)(int type, const char *usermask, const char *hostmask, SecurityGroup *match,
+                                        const char *reason, const char *setby,
+                                        time_t expire_at, time_t set_at, int soft, int flags);
 extern MODVAR TKL *(*tkl_add_banexception)(int type, const char *usermask, const char *hostmask, SecurityGroup *match,
                                            const char *reason, const char *set_by,
                                            time_t expire_at, time_t set_at, int soft, const char *bantypes, int flags);
@@ -820,6 +816,8 @@ extern MODVAR TKL *(*tkl_add_spamfilter)(int type, const char *id, unsigned shor
                                          const char *setby,
                                          time_t expire_at, time_t set_at,
                                          time_t spamf_tkl_duration, const char *spamf_tkl_reason,
+                                         int input_conversion,
+                                         SpamfilterShowMessageContentOnHit show_message_content_on_hit,
                                          int flags);
 extern MODVAR TKL *(*find_tkl_serverban)(int type, const char *usermask, const char *hostmask, int softban);
 extern MODVAR TKL *(*find_tkl_banexception)(int type, const char *usermask, const char *hostmask, int softban);
@@ -837,9 +835,9 @@ extern MODVAR TKL *(*find_qline)(Client *cptr, const char *nick, int *ishold);
 extern MODVAR TKL *(*find_tkline_match_zap)(Client *cptr);
 extern MODVAR void (*tkl_stats)(Client *cptr, int type, const char *para, int *cnt);
 extern MODVAR void (*tkl_sync)(Client *client);
-extern MODVAR void (*cmd_tkl)(Client *client, MessageTag *recv_mtags, int parc, const char *parv[]);
+extern MODVAR void (*cmd_tkl)(ClientContext *clictx, Client *client, MessageTag *recv_mtags, int parc, const char *parv[]);
 extern MODVAR int (*take_action)(Client *client, BanAction *actions, const char *reason, long duration, int take_action_flags, int *stopped);
-extern MODVAR int (*match_spamfilter)(Client *client, const char *str_in, int type, const char *cmd, const char *target, int flags, TKL **rettk);
+extern MODVAR int (*match_spamfilter)(Client *client, const char *str_in, int type, const char *cmd, const char *target, int flags, ClientContext *clictx, TKL **rettk);
 extern MODVAR int (*match_spamfilter_mtags)(Client *client, MessageTag *mtags, const char *cmd);
 extern MODVAR int (*join_viruschan)(Client *client, TKL *tk, int type);
 extern MODVAR const char *(*StripColors)(const char *text);
@@ -875,7 +873,7 @@ extern MODVAR void (*connect_server)(ConfigItem_link *aconf, Client *by, struct
 extern MODVAR int (*is_services_but_not_ulined)(Client *client);
 extern MODVAR void (*parse_message_tags)(Client *cptr, char **str, MessageTag **mtag_list);
 extern MODVAR const char *(*mtags_to_string)(MessageTag *m, Client *acptr);
-extern MODVAR int (*can_send_to_channel)(Client *cptr, Channel *channel, const char **msgtext, const char **errmsg, int notice);
+extern MODVAR int (*can_send_to_channel)(Client *cptr, Channel *channel, const char **msgtext, const char **errmsg, SendType sendtyp, ClientContext *clictx);
 extern MODVAR void (*broadcast_md_globalvar)(ModDataInfo *mdi, ModData *md);
 extern MODVAR void (*broadcast_md_globalvar_cmd)(Client *except, Client *sender, const char *varname, const char *value);
 extern MODVAR int (*tkl_ip_hash)(const char *ip);
@@ -922,14 +920,25 @@ extern MODVAR void (*cancel_ident_lookup)(Client *client);
 extern MODVAR int (*spamreport)(Client *client, const char *ip, NameValuePrioList *details, const char *spamreport_block, Client *by);
 extern MODVAR int (*crule_test)(const char *rule);
 extern MODVAR CRuleNode *(*crule_parse)(const char *rule);
-extern int (*crule_eval)(crule_context *context, CRuleNode *rule);
+extern MODVAR int (*crule_eval)(crule_context *context, CRuleNode *rule);
 #define safe_crule_free(x) do { if (x) crule_free(&x); } while(0)
-extern void (*crule_free)(CRuleNode **);
-extern const char *(*crule_errstring)(int errcode);
-extern void (*ban_act_set_reputation)(Client *client, BanAction *action);
-extern const char *(*get_central_api_key)(void);
-extern int (*central_spamreport)(Client *target, Client *by);
-extern int (*central_spamreport_enabled)(void);
+extern MODVAR void (*crule_free)(CRuleNode **);
+extern MODVAR const char *(*crule_errstring)(int errcode);
+extern MODVAR void (*ban_act_set_reputation)(Client *client, BanAction *action);
+extern MODVAR const char *(*get_central_api_key)(void);
+extern MODVAR int (*central_spamreport)(Client *target, Client *by, const char *url);
+extern MODVAR int (*central_spamreport_enabled)(void);
+extern MODVAR void (*sasl_succeeded)(Client *client);
+extern MODVAR void (*sasl_failed)(Client *client);
+extern MODVAR int (*decode_authenticate_plain)(const char *param, char **authorization_id, char **authentication_id, char **passwd);
+extern MODVAR void (*exit_client)(Client *client, MessageTag *recv_mtags, const char *comment);
+extern MODVAR void (*exit_client_fmt)(Client *client, MessageTag *recv_mtags, FORMAT_STRING(const char *pattern), ...) __attribute__((format(printf, 3, 4)));
+extern MODVAR void (*exit_client_ex)(Client *client, Client *origin, MessageTag *recv_mtags, const char *comment);
+extern MODVAR void (*banned_client)(Client *client, const char *bantype, const char *reason, int global, int noexit);
+extern MODVAR char *(*unreal_expand_string)(const char *str, char *buf, size_t buflen, NameValuePrioList *nvp, int buildvarstring_options, Client *client);
+extern MODVAR char *(*utf8_convert_confusables)(const char *i, char *obuf, int olen);
+extern MODVAR const char *(*utf8_get_block_name)(int i);
+extern MODVAR int (*utf8_get_block_number)(const char *name);
 /* /Efuncs */
 
 /* TLS functions */
@@ -950,9 +959,12 @@ extern int outdated_tls_client(Client *acptr);
 extern const char *outdated_tls_client_build_string(const char *pattern, Client *acptr);
 extern int check_certificate_expiry_ctx(SSL_CTX *ctx, char **errstr);
 extern EVENT(tls_check_expiry);
+extern int has_any_trusted_cert(void);
+extern int has_any_trusted_cert_with_correct_hostname(void);
 extern MODVAR EVP_MD *sha256_function;
 extern MODVAR EVP_MD *sha1_function;
 extern MODVAR EVP_MD *md5_function;
+extern int unrealircd_set_tls_groups(SSL_CTX *ctx, const char *groups);
 /* End of TLS functions */
 
 /* Default handlers for efunctions */
@@ -985,8 +997,14 @@ extern void cancel_ident_lookup_default_handler(Client *client);
 extern int spamreport_default_handler(Client *client, const char *ip, NameValuePrioList *details, const char *spamreport_block, Client *by);
 extern void ban_act_set_reputation_default_handler(Client *client, BanAction *action);
 extern const char *get_central_api_key_default_handler(void);
-extern int central_spamreport_default_handler(Client *target, Client *by);
+extern int central_spamreport_default_handler(Client *target, Client *by, const char *url);
 extern int central_spamreport_enabled_default_handler(void);
+extern void sasl_succeeded_default_handler(Client *client);
+extern void sasl_failed_default_handler(Client *client);
+extern int decode_authenticate_plain_default_handler(const char *param, char **authorization_id, char **authentication_id, char **passwd);
+extern char *utf8_convert_confusables_default_handler(const char *i, char *obuf, int olen);
+extern const char *utf8_get_block_name_default_handler(int i);
+extern int utf8_get_block_number_default_handler(const char *name);
 /* End of default handlers for efunctions */
 
 extern MODVAR MOTDFile opermotd, svsmotd, motd, botmotd, smotd, rules;
@@ -996,13 +1014,14 @@ extern int add_listmode_ex(Ban **list, Client *cptr, Channel *channel, const cha
 extern int del_listmode(Ban **list, Channel *channel, const char *banid);
 extern int Halfop_mode(long mode);
 extern const char *convert_regular_ban(char *mask, char *buf, size_t buflen);
-extern const char *clean_ban_mask(const char *, int, Client *, int);
+extern const char *clean_ban_mask(const char *, int, ExtbanType, Client *, Channel *, int);
 extern int find_invex(Channel *channel, Client *client);
 extern void DoMD5(char *mdout, const char *src, unsigned long n);
 extern char *md5hash(char *dst, const char *src, unsigned long n);
 extern char *sha256hash(char *dst, const char *src, unsigned long n);
 extern void sha256hash_binary(char *dst, const char *src, unsigned long n);
 extern void sha1hash_binary(char *dst, const char *src, unsigned long n);
+extern void binarytohex(void *data, size_t len, char *str);
 extern MODVAR TKL *tklines[TKLISTLEN];
 extern MODVAR TKL *tklines_ip_hash[TKLIPHASHLEN1][TKLIPHASHLEN2];
 extern const char *cmdname_by_spamftarget(int target);
@@ -1064,9 +1083,11 @@ extern int inet_pton6(const char *src, unsigned char *dst);
 extern const char *compressed_ip(const char *ip);
 extern int unreal_bind(int fd, const char *ip, int port, SocketType socket_type);
 extern int unreal_connect(int fd, const char *ip, int port, SocketType socket_type);
+extern const char *socket_type_valtostr(SocketType t);
 extern int is_valid_ip(const char *str);
 extern int ipv6_capable(void);
 extern int unix_sockets_capable(void);
+extern int set_client_ip(Client *client, const char *ip);
 #ifdef _WIN32
 extern void init_winsock(void);
 #endif
@@ -1096,7 +1117,7 @@ extern CMD_FUNC(cmd_module);
 extern CMD_FUNC(cmd_rehash);
 extern CMD_FUNC(cmd_die);
 extern CMD_FUNC(cmd_restart);
-extern void cmd_alias(Client *client, MessageTag *recv_mtags, int parc, const char *parv[], const char *cmd); /* special! */
+extern void cmd_alias(ClientContext *clictx, Client *client, MessageTag *recv_mtags, int parc, const char *parv[], const char *cmd); /* special! */
 extern const char *pcre2_version(void);
 extern int get_terminal_width(void);
 extern int has_common_channels(Client *c1, Client *c2);
@@ -1112,6 +1133,7 @@ extern char policy_valtochar(Policy policy);
 extern int verify_certificate(SSL *ssl, const char *hostname, char **errstr);
 extern const char *certificate_name(SSL *ssl);
 extern void start_of_normal_client_handshake(Client *acptr);
+extern int is_loopback_ip(char *ip);
 extern void clicap_pre_rehash(void);
 extern void clicap_check_for_changes(void);
 extern void unload_all_unused_mtag_handlers(void);
@@ -1123,7 +1145,6 @@ extern const char *spki_fingerprint_ex(X509 *x509_cert);
 extern int is_module_loaded(const char *name);
 extern int is_blacklisted_module(const char *name);
 extern void close_std_descriptors(void);
-extern void banned_client(Client *acptr, const char *bantype, const char *reason, int global, int noexit);
 extern char *mystpcpy(char *dst, const char *src);
 extern size_t add_sjsby(char *buf, const char *setby, time_t seton);
 extern MaxTarget *findmaxtarget(const char *cmd);
@@ -1195,6 +1216,7 @@ extern int is_extended_server_ban(const char *str);
 extern int empty_mode(const char *m);
 extern void free_multilinemode(MultiLineMode *m);
 #define safe_free_multilinemode(m) do { if (m) free_multilinemode(m); m = NULL; } while(0)
+extern ExtbanType mode_letter_to_extbantype(char c);
 extern int valid_sid(const char *name);
 extern int valid_uid(const char *name);
 extern void parse_client_queued(Client *client);
@@ -1212,7 +1234,6 @@ extern const char *unreal_add_quotes(const char *str);
 extern int unreal_add_quotes_r(const char *i, char *o, size_t len);
 extern void user_account_login(MessageTag *recv_mtags, Client *client);
 extern void link_generator(void);
-extern void update_throttling_timer_settings(void);
 extern int hide_idle_time(Client *client, Client *target);
 extern void lost_server_link(Client *serv, const char *tls_error_string);
 extern const char *sendtype_to_cmd(SendType sendtype);
@@ -1271,6 +1292,7 @@ extern void json_expand_client(json_t *j, const char *key, Client *client, int d
 extern void json_expand_client_security_groups(json_t *parent, Client *client);
 extern void json_expand_channel(json_t *j, const char *key, Channel *channel, int detail);
 extern void json_expand_tkl(json_t *j, const char *key, TKL *tkl, int detail);
+extern void json_expand_textanalysis(json_t *root, const char *key, TextAnalysis *ta, int detail);
 /* end of json.c */
 /* securitygroup.c start */
 extern MODVAR SecurityGroup *securitygroups;
@@ -1288,9 +1310,11 @@ extern int security_group_exists(const char *name);
 extern SecurityGroup *add_security_group(const char *name, int order);
 extern SecurityGroup *find_security_group(const char *name);
 extern void free_security_group(SecurityGroup *s);
+#define safe_free_security_group(x) do { if (x) { free_security_group(x); x = NULL; } } while(0)
 extern SecurityGroup *duplicate_security_group(SecurityGroup *s);
 extern void set_security_group_defaults(void);
 extern int user_allowed_by_security_group(Client *client, SecurityGroup *s);
+extern int user_allowed_by_security_group_context(Client *client, SecurityGroup *s, crule_context *context);
 extern int user_allowed_by_security_group_name(Client *client, const char *secgroupname);
 extern const char *get_security_groups(Client *client);
 extern int test_match_item(ConfigFile *conf, ConfigEntry *cep, int *errors);
@@ -1300,6 +1324,8 @@ extern int conf_match_block(ConfigFile *conf, ConfigEntry *ce, SecurityGroup **b
 extern int test_extended_list(Extban *extban, ConfigEntry *cep, int *errors);
 extern int test_set_security_group(ConfigFile *conf, ConfigEntry *ce);
 extern int config_set_security_group(ConfigFile *conf, ConfigEntry *ce);
+extern int user_matches_extended_server_ban(Client *client, const char *name, const char *value);
+extern int user_matches_extended_list(Client *client, NameValuePrioList *e);
 /* securitygroup.c end */
 /* src/unrealdb.c start */
 extern UnrealDB *unrealdb_open(const char *filename, UnrealDBMode mode, char *secret_block);
@@ -1343,8 +1369,16 @@ extern const char *log_level_terminal_color(LogLevel loglevel);
 extern LogType log_type_stringtoval(const char *str);
 extern const char *log_type_valtostring(LogType v);
 #ifdef DEBUGMODE
-#define unreal_log(...) do_unreal_log(__VA_ARGS__, log_data_source(__FILE__, __LINE__, __FUNCTION__), NULL)
-#define unreal_log_raw(...) do_unreal_log_raw(__VA_ARGS__, log_data_source(__FILE__, __LINE__, __FUNCTION__), NULL)
+/* In debug mode we include file/linenumber. We put this arg at the end, however
+ * there is an issue if unreal_log() is used with a parameter like xyz ? log_data_string("zzz") : NULL,
+ * since then our log_data_source() would be beyond NULL and thus would never be freed,
+ * so we allocate and handle that differently. File/line would still be lost but at
+ * least there is no memory leak. Alternative solution is to specify first couple of
+ * parameters explicitly, put log_data_source() at the beginning of the argument list
+ * and then use non-portable ## __VA_ARGS__ for the remainder.
+ */
+#define unreal_log(...) do { LogData *lds = log_data_source(__FILE__, __LINE__, __FUNCTION__); do_unreal_log(__VA_ARGS__, lds, NULL); log_data_free(lds); } while(0)
+#define unreal_log_raw(...) do { LogData *lds = log_data_source(__FILE__, __LINE__, __FUNCTION__); do_unreal_log_raw(__VA_ARGS__, lds, NULL); log_data_free(lds); } while(0)
 #else
 #define unreal_log(...) do_unreal_log(__VA_ARGS__, NULL)
 #define unreal_log_raw(...) do_unreal_log_raw(__VA_ARGS__, NULL)
@@ -1362,7 +1396,9 @@ extern LogData *log_data_source(const char *file, int line, const char *function
 extern LogData *log_data_socket_error(int fd);
 extern LogData *log_data_link_block(ConfigItem_link *link);
 extern LogData *log_data_tkl(const char *key, TKL *tkl);
+extern LogData *log_data_textanalysis(const char *key, TextAnalysis *ta);
 extern LogData *log_data_tls_error(void);
+extern void log_data_free(LogData *d);
 extern void log_pre_rehash(void);
 extern int log_tests(void);
 extern void config_pre_run_log(void);
@@ -1396,7 +1432,6 @@ extern GeoIPResult *geoip_lookup(const char *ip);
 extern void free_geoip_result(GeoIPResult *r);
 extern const char *get_operlogin(Client *client);
 extern const char *get_operclass(Client *client);
-extern struct sockaddr *raw_client_ip(Client *client);
 /* url stuff */
 extern const char *unreal_mkcache(const char *url);
 extern int has_cached_version(const char *url);
