@@ -30,8 +30,6 @@
 
 #include "unrealircd.h"
 
-static void exit_one_client(Client *, MessageTag *mtags_i, const char *);
-
 static const char *months[] = {
 	"January", "February", "March", "April",
 	"May", "June", "July", "August",
@@ -363,317 +361,6 @@ int on_dccallow_list(Client *to, Client *from)
 	return 0;
 }
 
-/** Delete all DCCALLOW references.
- * Ultimately, this should be moved to modules/dccallow.c
- */
-void remove_dcc_references(Client *client)
-{
-	Client *acptr;
-	Link *lp, *nextlp;
-	Link **lpp, *tmp;
-	int found;
-
-	lp = client->user->dccallow;
-	while(lp)
-	{
-		nextlp = lp->next;
-		acptr = lp->value.client;
-		for(found = 0, lpp = &(acptr->user->dccallow); *lpp; lpp=&((*lpp)->next))
-		{
-			if (lp->flags == (*lpp)->flags)
-				continue; /* match only opposite types for sanity */
-			if ((*lpp)->value.client == client)
-			{
-				if ((*lpp)->flags == DCC_LINK_ME)
-				{
-					sendto_one(acptr, NULL, ":%s %d %s :%s has been removed from "
-						"your DCC allow list for signing off",
-						me.name, RPL_DCCINFO, acptr->name, client->name);
-				}
-				tmp = *lpp;
-				*lpp = tmp->next;
-				free_link(tmp);
-				found++;
-				break;
-			}
-		}
-
-		if (!found)
-		{
-			unreal_log(ULOG_WARNING, "main", "BUG_REMOVE_DCC_REFERENCES", acptr,
-			           "[BUG] remove_dcc_references: $client was in dccallowme "
-			           "list of $existing_client but not in dccallowrem list!",
-			           log_data_client("existing_client", client));
-		}
-
-		free_link(lp);
-		lp = nextlp;
-	}
-}
-
-/*
- * Remove all clients that depend on source_p; assumes all (S)QUITs have
- * already been sent.  we make sure to exit a server's dependent clients
- * and servers before the server itself; exit_one_client takes care of
- * actually removing things off llists.   tweaked from +CSr31  -orabidoo
- */
-static void recurse_remove_clients(Client *client, MessageTag *mtags, const char *comment)
-{
-	Client *acptr, *next;
-
-	list_for_each_entry_safe(acptr, next, &client_list, client_node)
-	{
-		if (acptr->uplink != client)
-			continue;
-
-		exit_one_client(acptr, mtags, comment);
-	}
-
-	list_for_each_entry_safe(acptr, next, &global_server_list, client_node)
-	{
-		if (acptr->uplink != client)
-			continue;
-
-		recurse_remove_clients(acptr, mtags, comment);
-		exit_one_client(acptr, mtags, comment);
-	}
-}
-
-/*
-** Remove *everything* that depends on source_p, from all lists, and sending
-** all necessary QUITs and SQUITs.  source_p itself is still on the lists,
-** and its SQUITs have been sent except for the upstream one  -orabidoo
-*/
-static void remove_dependents(Client *client, Client *from, MessageTag *mtags, const char *comment, const char *splitstr)
-{
-	Client *acptr;
-
-	list_for_each_entry(acptr, &global_server_list, client_node)
-	{
-		if (acptr != from && !(acptr->direction && (acptr->direction == from)))
-			sendto_one(acptr, mtags, "SQUIT %s :%s", client->name, comment);
-	}
-
-	recurse_remove_clients(client, mtags, splitstr);
-}
-
-/*
-** Exit one client, local or remote. Assuming all dependants have
-** been already removed, and socket closed for local client.
-*/
-static void exit_one_client(Client *client, MessageTag *mtags_i, const char *comment)
-{
-	Link *lp;
-	Membership *mp;
-
-	assert(!IsMe(client));
-
-	if (IsUser(client))
-	{
-		MessageTag *mtags_o = NULL;
-
-		if (!MyUser(client))
-			RunHook(HOOKTYPE_REMOTE_QUIT, client, mtags_i, comment);
-
-		new_message_special(client, mtags_i, &mtags_o, ":%s QUIT", client->name);
-		if (find_mtag(mtags_o, "unrealircd.org/real-quit-reason"))
-			quit_sendto_local_common_channels(client, mtags_o, comment);
-		else
-			sendto_local_common_channels(client, NULL, 0, mtags_o, ":%s QUIT :%s", client->name, comment);
-		free_message_tags(mtags_o);
-
-		while ((mp = client->user->channel))
-			remove_user_from_channel(client, mp->channel, 1);
-		/* again, this is all that is needed */
-
-		/* Clean up dccallow list and (if needed) notify other clients
-		 * that have this person on DCCALLOW that the user just left/got removed.
-		 */
-		remove_dcc_references(client);
-
-		/* For remote clients, we need to check for any outstanding async
-		 * connects attached to this 'client', and set those records to NULL.
-		 * Why not for local? Well, we already do that in close_connection ;)
-		 */
-		if (!MyConnect(client))
-			unrealdns_delreq_bycptr(client);
-	}
-
-	/* Free module related data for this client */
-	moddata_free_client(client);
-	if (MyConnect(client))
-		moddata_free_local_client(client);
-
-	/* Remove client from the client list */
-	if (*client->id)
-	{
-		del_from_id_hash_table(client->id, client);
-		*client->id = '\0';
-	}
-	if (*client->name)
-		del_from_client_hash_table(client->name, client);
-	if (remote_rehash_client == client)
-		remote_rehash_client = NULL; /* client did a /REHASH and QUIT before rehash was complete */
-	remove_client_from_list(client);
-}
-
-/** Exit this IRC client, and all the dependents (users, servers) if this is a server.
- * @param client        The client to exit.
- * @param recv_mtags  Message tags to use as a base (if any).
- * @param comment     The (s)quit message
- */
-void exit_client(Client *client, MessageTag *recv_mtags, const char *comment)
-{
-	exit_client_ex(client, client->direction, recv_mtags, comment);
-}
-
-/** Exit this IRC client, and all the dependents (users, servers) if this is a server.
- * @param client        The client to exit.
- * @param recv_mtags  Message tags to use as a base (if any).
- * @param comment     The (s)quit message
- */
-void exit_client_fmt(Client *client, MessageTag *recv_mtags, FORMAT_STRING(const char *pattern), ...)
-{
-	char comment[512];
-
-	va_list vl;
-	va_start(vl, pattern);
-	vsnprintf(comment, sizeof(comment), pattern, vl);
-	va_end(vl);
-
-	exit_client_ex(client, client->direction, recv_mtags, comment);
-}
-
-/** Exit this IRC client, and all the dependents (users, servers) if this is a server.
- * @param client        The client to exit.
- * @param recv_mtags  Message tags to use as a base (if any).
- * @param comment     The (s)quit message
- */
-void exit_client_ex(Client *client, Client *origin, MessageTag *recv_mtags, const char *comment)
-{
-	long long on_for;
-	ConfigItem_listen *listen_conf;
-	MessageTag *mtags_generated = NULL;
-
-	if (IsDead(client))
-		return; /* Already marked as exited */
-
-	/* We replace 'recv_mtags' here with a newly
-	 * generated id if 'recv_mtags' is NULL or is
-	 * non-NULL and contains no msgid etc.
-	 * This saves us from doing a new_message()
-	 * prior to the exit_client() call at around
-	 * 100+ places elsewhere in the code.
-	 */
-	new_message(client, recv_mtags, &mtags_generated);
-	recv_mtags = mtags_generated;
-
-	if (MyConnect(client))
-	{
-		if (client->local->class)
-		{
-			client->local->class->clients--;
-			if ((client->local->class->flag.temporary) && !client->local->class->clients && !client->local->class->xrefcount)
-			{
-				delete_classblock(client->local->class);
-				client->local->class = NULL;
-			}
-		}
-		if (IsUser(client))
-			irccounts.me_clients--;
-		if (client->server && client->server->conf)
-		{
-			client->server->conf->refcount--;
-			if (!client->server->conf->refcount
-			  && client->server->conf->flag.temporary)
-			{
-				delete_linkblock(client->server->conf);
-				client->server->conf = NULL;
-			}
-		}
-		if (IsServer(client))
-		{
-			irccounts.me_servers--;
-			if (!IsServerDisconnectLogged(client))
-			{
-				unreal_log(ULOG_ERROR, "link", "LINK_DISCONNECTED", client,
-					   "Lost server link to $client [$client.ip]: $reason",
-					   log_data_string("reason", comment));
-			}
-		}
-		free_pending_net(client);
-		SetClosing(client);
-		if (IsUser(client))
-		{
-			long connected_time = TStime() - client->local->creationtime;
-			RunHook(HOOKTYPE_LOCAL_QUIT, client, recv_mtags, comment);
-			unreal_log(ULOG_INFO, "connect", "LOCAL_CLIENT_DISCONNECT", client,
-				   "Client exiting: $client ($client.user.username@$client.hostname) [$client.ip] ($reason)",
-				   log_data_string("extended_client_info", get_connect_extinfo(client)),
-				   log_data_string("reason", comment),
-				   log_data_integer("connected_time", connected_time));
-		} else
-		if (IsUnknown(client))
-		{
-			RunHook(HOOKTYPE_UNKUSER_QUIT, client, recv_mtags, comment);
-		}
-
-		if (client->local->fd >= 0 && !IsConnecting(client))
-		{
-			if (!IsControl(client) && !IsRPC(client))
-				sendto_one(client, NULL, "ERROR :Closing Link: %s (%s)", get_client_name(client, FALSE), comment);
-		}
-		close_connection(client);
-	}
-	else if (IsUser(client) && !IsULine(client))
-	{
-		if (client->uplink != &me)
-		{
-			unreal_log(ULOG_INFO, "connect", "REMOTE_CLIENT_DISCONNECT", client,
-				   "Client exiting: $client ($client.user.username@$client.hostname) [$client.ip] ($reason)",
-				   log_data_string("extended_client_info", get_connect_extinfo(client)),
-				   log_data_string("reason", comment),
-				   log_data_string("from_server_name", client->user->server));
-		}
-	}
-
-	/*
-	 * Recurse down the client list and get rid of clients who are no
-	 * longer connected to the network (from my point of view)
-	 * Only do this expensive stuff if exited==server -Donwulff
-	 */
-	if (IsServer(client))
-	{
-		char splitstr[HOSTLEN + HOSTLEN + 2];
-		Client *acptr, *next;
-
-		assert(client->server != NULL && client->uplink != NULL);
-
-		if (FLAT_MAP)
-			strlcpy(splitstr, "*.net *.split", sizeof splitstr);
-		else
-			ircsnprintf(splitstr, sizeof splitstr, "%s %s", client->uplink->name, client->name);
-
-		remove_dependents(client, origin, recv_mtags, comment, splitstr);
-
-		/* Special case for remote async RPC, server.rehash in particular.. */
-		list_for_each_entry_safe(acptr, next, &rpc_remote_list, client_node)
-			if (!strncmp(client->id, acptr->id, SIDLEN))
-				free_client(acptr);
-
-		RunHook(HOOKTYPE_SERVER_QUIT, client, recv_mtags);
-	}
-	else if (IsUser(client) && !IsKilled(client))
-	{
-		sendto_server(client, 0, 0, recv_mtags, ":%s QUIT :%s", client->id, comment);
-	}
-
-	/* Finally, the client/server itself exits.. */
-	exit_one_client(client, recv_mtags, comment);
-
-	free_message_tags(mtags_generated);
-}
-
 /** Initialize the (quite useless) IRC statistics */
 void initstats(void)
 {
@@ -784,6 +471,22 @@ int valid_vhost(const char *userhost)
 	return 1;
 }
 
+/** Weird variant of valid_vhost() that allows $varnames through.
+ * So should only be used for checking things like vhost::vhost,
+ * oper::vhost and such in the CONFIG TEST routines.
+ * At runtime you should use valid_vhost() to check the final host
+ * again after variable expansion (by unreal_expand_string usually).
+ */
+int potentially_valid_vhost(const char *userhost)
+{
+	char uhost[512], *p;
+	strlcpy(uhost, userhost, sizeof(uhost));
+	for (p = uhost; *p; p++)
+		if (*p == '$')
+			*p = '_';
+	return valid_vhost(uhost);
+}
+
 /*|| BAN ACTION ROUTINES FOLLOW ||*/
 
 int parse_ban_action_set(const char *str, char **var, VarActionValue *op, int *value, char **error)
@@ -1285,6 +988,20 @@ const char *cmdname_by_spamftarget(int target)
 	return "???";
 }
 
+/** Convert the value for set::spamfilter::show-message-content-on-hit
+ * and spamfilter::show-message-content-on-hit.
+ */
+SpamfilterShowMessageContentOnHit spamfilter_show_message_content_on_hit_strtoval(const char *s)
+{
+	if (!strcmp(s, "always"))
+		return SPAMFILTER_SHOW_MESSAGE_CONTENT_ON_HIT_ALWAYS;
+	if (!strcmp(s, "channel-only"))
+		return SPAMFILTER_SHOW_MESSAGE_CONTENT_ON_HIT_CHANNEL_ONLY;
+	if (!strcmp(s, "never"))
+		return SPAMFILTER_SHOW_MESSAGE_CONTENT_ON_HIT_NEVER;
+	return 0;
+}
+
 /** Add name entries from config */
 void unreal_add_names(NameList **n, ConfigEntry *ce)
 {
@@ -1478,109 +1195,6 @@ const char *compressed_ip(const char *ip)
 	return ip;
 }
 
-static int should_hide_ban_reason(Client *client, const char *reason)
-{
-	if (HIDE_BAN_REASON == HIDE_BAN_REASON_AUTO)
-	{
-		/* If we detect the IP address in the ban reason or
-		 * it contains an unrealircd.org/ URL then the
-		 * ban reason is hidden since it may expose client
-		 * details.
-		 */
-		// First the simple check:
-		if (strstr(reason, "unrealircd.org/") ||
-		    strstr(reason, client->ip))
-		{
-			return 1;
-		}
-		// For IPv6, check compressed IP too:
-		if (IsIPV6(client))
-		{
-			const char *ip = compressed_ip(client->ip);
-			if (strstr(reason, ip))
-				return 1;
-		}
-		return 0;
-	} else {
-		return HIDE_BAN_REASON == HIDE_BAN_REASON_YES ? 1 : 0;
-	}
-}
-
-/** Generic function to inform the user he/she has been banned.
- * @param client   The affected client.
- * @param bantype  The ban type, such as: "K-Lined", "G-Lined" or "realname".
- * @param reason   The specified reason.
- * @param global   Whether the ban is global (1) or for this server only (0)
- * @param noexit   Set this to NO_EXIT_CLIENT to make us not call exit_client().
- *                 This is really only needed from the accept code, do not
- *                 use it anywhere else. No really, never.
- *
- * @note This function will call exit_client() appropriately.
- */
-void banned_client(Client *client, const char *bantype, const char *reason, int global, int noexit)
-{
-	char buf[512];
-	char *fmt = global ? iConf.reject_message_gline : iConf.reject_message_kline;
-	const char *vars[6], *values[6];
-	MessageTag *mtags = NULL;
-
-	if (!MyConnect(client))
-		abort();
-
-	/* This was: "You are not welcome on this %s. %s: %s. %s" but is now dynamic: */
-	vars[0] = "bantype";
-	values[0] = bantype;
-	vars[1] = "banreason";
-	values[1] = reason;
-	vars[2] = "klineaddr";
-	values[2] = KLINE_ADDRESS;
-	vars[3] = "glineaddr";
-	values[3] = GLINE_ADDRESS ? GLINE_ADDRESS : KLINE_ADDRESS; /* fallback to klineaddr */
-	vars[4] = "ip";
-	values[4] = GetIP(client);
-	vars[5] = NULL;
-	values[5] = NULL;
-	buildvarstring(fmt, buf, sizeof(buf), vars, values);
-
-	/* This is a bit extensive but we will send both a YOUAREBANNEDCREEP
-	 * and a notice to the user.
-	 * The YOUAREBANNEDCREEP will be helpful for the client since it makes
-	 * clear the user should not quickly reconnect, as (s)he is banned.
-	 * The notice still needs to be there because it stands out well
-	 * at most IRC clients.
-	 */
-	if (noexit != NO_EXIT_CLIENT)
-	{
-		sendnumeric(client, ERR_YOUREBANNEDCREEP, buf);
-		sendnotice(client, "%s", buf);
-	}
-
-	/* The final message in the ERROR is shorter. */
-	if (IsRegistered(client) && should_hide_ban_reason(client, reason))
-	{
-		/* Hide the ban reason, but put the real reason in unrealircd.org/real-quit-reason */
-		MessageTag *m = safe_alloc(sizeof(MessageTag));
-		safe_strdup(m->name, "unrealircd.org/real-quit-reason");
-		snprintf(buf, sizeof(buf), "Banned (%s): %s", bantype, reason);
-		safe_strdup(m->value, buf);
-		AddListItem(m, mtags);
-		/* And the quit reason for anyone else, goes here.. */
-		snprintf(buf, sizeof(buf), "Banned (%s)", bantype);
-	} else {
-		snprintf(buf, sizeof(buf), "Banned (%s): %s", bantype, reason);
-	}
-
-	if (noexit != NO_EXIT_CLIENT)
-	{
-		exit_client(client, mtags, buf);
-	} else {
-		/* Special handling for direct Z-line code */
-		client->flags |= CLIENT_FLAG_DEADSOCKET_IS_BANNED;
-		dead_socket(client, buf);
-	}
-	safe_free_message_tags(mtags);
-}
-
 /** Our stpcpy implementation - discouraged due to lack of bounds checking */
 char *mystpcpy(char *dst, const char *src)
 {
@@ -1899,7 +1513,7 @@ const char *get_central_api_key_default_handler(void)
 	return NULL;
 }
 
-int central_spamreport_default_handler(Client *target, Client *by)
+int central_spamreport_default_handler(Client *target, Client *by, const char *url)
 {
 	return 0;
 }
@@ -1909,6 +1523,35 @@ int central_spamreport_enabled_default_handler(void)
 	return 0;
 }
 
+void sasl_succeeded_default_handler(Client *client)
+{
+}
+
+void sasl_failed_default_handler(Client *client)
+{
+}
+
+int decode_authenticate_plain_default_handler(const char *param, char **authorization_id, char **authentication_id, char **passwd)
+{
+	return 0;
+}
+
+char *utf8_convert_confusables_default_handler(const char *i, char *obuf, int olen)
+{
+	strlcpy(obuf, i, olen);
+	return obuf;
+}
+
+const char *utf8_get_block_name_default_handler(int i)
+{
+	return NULL;
+}
+
+int utf8_get_block_number_default_handler(const char *name)
+{
+	return -1;
+}
+
 /** my_timegm: mktime()-like function which will use GMT/UTC.
  * Strangely enough there is no standard function for this.
  * On some *NIX OS's timegm() may be available, sometimes only
@@ -2220,7 +1863,17 @@ void DoMD5(char *mdout, const char *src, unsigned long n)
 {
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
 	unsigned int md_len;
-	EVP_MD_CTX *mdctx = EVP_MD_CTX_new();
+	EVP_MD_CTX *mdctx;
+
+	if (!md5_function)
+	{
+		unreal_log(ULOG_FATAL, "tls", "MD5_UNAVAILABLE_FATAL", NULL,
+		           "DoMD5() was called but the MD5 algorithm is not available "
+		           "in your OpenSSL/LibreSSL version. -- ABORTING");
+		abort();
+	}
+
+	mdctx = EVP_MD_CTX_new();
 	if (EVP_DigestInit_ex(mdctx, md5_function, NULL) != 1)
 		abort();
 	EVP_DigestUpdate(mdctx, src, n);
