@@ -8,7 +8,7 @@
 ModuleHeader MOD_HEADER
   = {
 	"central-blocklist",
-	"1.0.6",
+	"1.0.8",
 	"Check users at central blocklist",
 	"UnrealIRCd Team",
 	"unrealircd-6",
@@ -20,7 +20,10 @@ Module *cbl_module = NULL;
 #define CBL_URL	 "https://centralblocklist.unrealircd-api.org/api/v1"
 #define SPAMREPORT_URL	"https://spamreport.unrealircd-api.org/api/spamreport-v1"
 #define CBL_TRANSFER_TIMEOUT 10
-#define SPAMREPORT_NUM_REMEMBERED_CMDS 10
+#define SPAMREPORT_NUM_REMEMBERED_CMDS 20
+
+#define WEB(client)		((WebRequest *)moddata_client(client, webserver_md).ptr)
+#define WSU(client)		((WebSocketUser *)moddata_client(client, websocket_md).ptr)
 
 typedef struct CBLUser CBLUser;
 struct CBLUser
@@ -31,6 +34,7 @@ struct CBLUser
 	char allowed_in;
 	int last_cmds_slot;
 	char *last_cmds[SPAMREPORT_NUM_REMEMBERED_CMDS];
+	TextAnalysis last_cmds_textanalysis[SPAMREPORT_NUM_REMEMBERED_CMDS];
 };
 
 /* For tracking current HTTPS requests */
@@ -70,9 +74,11 @@ struct reqstruct {
 static struct reqstruct req;
 
 CBLTransfer *cbltransfers = NULL;
+ModDataInfo *webserver_md = NULL; /* (external module, looked up) */
+ModDataInfo *websocket_md = NULL; /* (external module, looked up) */
 
 /* Forward declarations */
-int _central_spamreport(Client *client, Client *by);
+int _central_spamreport(Client *client, Client *by, const char *url);
 int cbl_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs);
 int cbl_config_posttest(int *errs);
 int cbl_config_run(ConfigFile *cf, ConfigEntry *ce, int type);
@@ -244,7 +250,10 @@ MOD_LOAD()
 
 	do_command_overrides(modinfo);
 
-	/* Enable gathering of "last 10 lines" for SPAMREPORT, only if SPAMREPORT is enabled: */
+	webserver_md = findmoddata_byname("web", MODDATATYPE_CLIENT);
+	websocket_md = findmoddata_byname("websocket", MODDATATYPE_CLIENT);
+
+	/* Enable gathering of "last 20 lines" for SPAMREPORT, only if SPAMREPORT is enabled: */
 	if (central_spamreport_enabled())
 	{
 		CommandOverrideAdd(modinfo->handle, "NICK", -2, cbl_override_spamreport_gather);
@@ -617,6 +626,28 @@ void cbl_add_client_info(Client *client)
 		json_object_set_new(user, "reputation", json_integer(GetReputation(client)));
 	}
 
+	if (webserver_md && WEB(client))
+	{
+		json_t *web = json_object();
+		json_t *headers = json_object();
+		NameValuePrioList *nv;
+		json_object_set_new(child, "web", web);
+		json_object_set_new(web, "headers", headers);
+		for (nv = WEB(client)->headers; nv; nv = nv->next)
+			json_object_set_new(headers, nv->name, json_string_unreal(nv->value));
+	}
+
+	if (websocket_md && WSU(client))
+	{
+		json_t *websocket = json_object();
+		json_object_set_new(child, "websocket", websocket);
+
+		if (WSU(client)->type == WEBSOCKET_TYPE_TEXT)
+			json_object_set_new(websocket, "protocol", json_string_unreal("text"));
+		else if (WSU(client)->type == WEBSOCKET_TYPE_BINARY)
+			json_object_set_new(websocket, "protocol", json_string_unreal("binary"));
+	}
+
 	if ((str = moddata_client_get(client, "tls_cipher")))
 	{
 		json_t *tls = json_object();
@@ -773,7 +804,11 @@ int cbl_is_handshake_finished(Client *client)
 void cbl_allow(Client *client)
 {
 	if (CBL(client))
+	{
+		if (CBL(client)->allowed_in)
+			return; /* Already allowed in */
 		CBL(client)->allowed_in = 1;
+	}
 
 	if (is_handshake_finished(client))
 		register_user(client);
@@ -850,6 +885,8 @@ void cbl_error_response(CBLTransfer *transfer, const char *error)
 		client = hash_find_id(n->name, NULL);
 		if (!client)
 			continue; /* Client disconnected already */
+		if (CBL(client) && CBL(client)->allowed_in)
+			continue; /* Client allowed in already (eg due to timeout) */
 		unreal_log(ULOG_DEBUG, "central-blocklist", "DEBUG_CENTRAL_BLOCKLIST_ERROR", client,
 			   "CBL: Client $client.details allowed in due to CBL error: $error",
 			   log_data_string("error", error));
@@ -886,10 +923,12 @@ void cbl_download_complete(OutgoingWebRequest *request, OutgoingWebResponse *res
 
 	if (response->errorbuf || !response->memory)
 	{
+		char buf[512];
 		unreal_log(ULOG_DEBUG, "central-blocklist", "DEBUG_CENTRAL_BLOCKLIST", NULL,
 		           "CBL ERROR: $error",
 		           log_data_string("error", response->errorbuf ? response->errorbuf : "No data returned"));
-		cbl_error_response(transfer, "error contacting CBL");
+		snprintf(buf, sizeof(buf), "error contacting CBL: %s", response->errorbuf ? response->errorbuf : "No data returned");
+		cbl_error_response(transfer, buf);
 		return;
 	}
 
@@ -1066,7 +1105,14 @@ CMD_OVERRIDE_FUNC(cbl_override_spamreport_gather)
 		}
 		if (record_cmd)
 		{
-			safe_strdup(CBL(client)->last_cmds[CBL(client)->last_cmds_slot], backupbuf);
+			int slot = CBL(client)->last_cmds_slot; // just for readability below
+			safe_strdup(CBL(client)->last_cmds[slot], backupbuf);
+			if (clictx && clictx->textanalysis)
+			{
+				memcpy(&CBL(client)->last_cmds_textanalysis[slot], clictx->textanalysis, sizeof(TextAnalysis));
+			} else {
+				memset(&CBL(client)->last_cmds_textanalysis[slot], 0, sizeof(TextAnalysis));
+			}
 			CBL(client)->last_cmds_slot++;
 			if (CBL(client)->last_cmds_slot >= SPAMREPORT_NUM_REMEMBERED_CMDS)
 				CBL(client)->last_cmds_slot = 0;
@@ -1076,7 +1122,7 @@ CMD_OVERRIDE_FUNC(cbl_override_spamreport_gather)
 	CALL_NEXT_COMMAND_OVERRIDE();
 }
 
-int _central_spamreport(Client *client, Client *by)
+int _central_spamreport(Client *client, Client *by, const char *url)
 {
 	json_t *j, *requests, *data, *cmds, *item;
 	OutgoingWebRequest *w;
@@ -1087,6 +1133,9 @@ int _central_spamreport(Client *client, Client *by)
 	char number[16];
 	int cnt = 0;
 
+	if (!client)
+		return 0; /* We only support reporting clients, not clientless IP addresses */
+
 	if (!MyUser(client) || !CBL(client))
 		return 0; /* Only possible if hot-loading */
 
@@ -1117,9 +1166,12 @@ int _central_spamreport(Client *client, Client *by)
 	{
 		if (CBL(client)->last_cmds[i])
 		{
+			// WARNING: duplicate code #1 of #2
 			snprintf(number, sizeof(number), "%d", ++cnt);
 			item = json_object();
 			json_object_set_new(item, "raw", json_string_unreal(CBL(client)->last_cmds[i]));
+			if (CBL(client)->last_cmds_textanalysis[i].num_bytes)
+				json_expand_textanalysis(item, "textanalysis", &CBL(client)->last_cmds_textanalysis[i], 2);
 			json_object_set_new(cmds, number, item);
 		}
 	}
@@ -1127,9 +1179,12 @@ int _central_spamreport(Client *client, Client *by)
 	{
 		if (CBL(client)->last_cmds[i])
 		{
+			// WARNING: duplicate code #2 of #2
 			snprintf(number, sizeof(number), "%d", ++cnt);
 			item = json_object();
 			json_object_set_new(item, "raw", json_string_unreal(CBL(client)->last_cmds[i]));
+			if (CBL(client)->last_cmds_textanalysis[i].num_bytes)
+				json_expand_textanalysis(item, "textanalysis", &CBL(client)->last_cmds_textanalysis[i], 2);
 			json_object_set_new(cmds, number, item);
 		}
 	}
@@ -1148,7 +1203,7 @@ int _central_spamreport(Client *client, Client *by)
 	add_nvplist(&headers, 0, "X-API-Key", cfg.api_key);
 	/* Do the web request */
 	w = safe_alloc(sizeof(OutgoingWebRequest));
-	safe_strdup(w->url, cfg.spamreport_url);
+	safe_strdup(w->url, url ? url : cfg.spamreport_url);
 	w->http_method = HTTP_METHOD_POST;
 	w->body = json_serialized;
 	w->headers = headers;
