@@ -165,7 +165,7 @@ int floodprot_join(Client *client, Channel *channel, MessageTag *mtags);
 EVENT(modef_event);
 int cmodef_channel_create(Channel *channel);
 int cmodef_channel_destroy(Channel *channel, int *should_destroy);
-int floodprot_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype);
+int floodprot_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype, ClientContext *clictx);
 int floodprot_post_chanmsg(Client *client, Channel *channel, int sendflags, const char *prefix, const char *target, MessageTag *mtags, const char *text, SendType sendtype);
 int floodprot_knock(Client *client, Channel *channel, MessageTag *mtags, const char *comment);
 int floodprot_nickchange(Client *client, MessageTag *mtags, const char *oldnick);
@@ -294,6 +294,9 @@ MOD_UNLOAD()
 
 	free_channel_flood_profiles();
 
+	/* Free config */
+	safe_free(cfg.default_profile);
+
 	return MOD_SUCCESS;
 }
 
@@ -369,6 +372,7 @@ static void init_config(void)
 	cfg.modef_alternate_action_percentage_threshold = 75; /* 75% */
 	cfg.modef_alternative_ban_action_unsettime = 15; /* 15min */
 	init_default_channel_flood_profiles();
+	safe_strdup(cfg.default_profile, "normal"); // use profile "normal" by default (U6.2.0+)
 }
 
 int floodprot_config_test_set_block(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
@@ -580,6 +584,9 @@ int floodprot_config_run_antiflood_block(ConfigFile *cf, ConfigEntry *ce, int ty
 		if (!strcmp(ce->name, "default-profile"))
 		{
 			safe_strdup(cfg.default_profile, ce->value);
+			/* Let's handle 'off' in a special way -> becomes NULL */
+			if (!strcmp(cfg.default_profile, "off"))
+				safe_free(cfg.default_profile);
 		} else
 		if (!strcmp(ce->name, "boot-delay"))
 		{
@@ -1120,6 +1127,7 @@ int is_floodprot_exempt(Client *client, Channel *channel, char flood_type_letter
 	char *p;
 	BanContext *b = safe_alloc(sizeof(BanContext));
 
+	b->ban_type = EXBTYPE_EXCEPT;
 	b->client = client;
 	b->channel = channel;
 	b->ban_check_types = BANCHK_MSG;
@@ -1284,7 +1292,7 @@ ChannelFloodProtection *get_channel_flood_settings(Channel *channel, int what)
 	return NULL;
 }
 
-int floodprot_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype)
+int floodprot_can_send_to_channel(Client *client, Channel *channel, Membership *lp, const char **msg, const char **errmsg, SendType sendtype, ClientContext *clictx)
 {
 	Membership *mb;
 	ChannelFloodProtection *fld;
