@@ -45,7 +45,6 @@ ModuleHeader MOD_HEADER
  */
 #define ASSUME_NICK_IN_FLIGHT
 
-/* Variables */
 static char spamfilter_user[NICKLEN + USERLEN + HOSTLEN + REALLEN + 64];
 
 /* Forward declarations */
@@ -56,20 +55,23 @@ CMD_FUNC(cmd_uid);
 int _register_user(Client *client);
 void nick_collision(Client *cptr, const char *newnick, const char *newid, Client *new, Client *existing, int type);
 int AllowClient(Client *client);
-int exceeds_maxperip(Client *client, ConfigItem_allow *aconf);
+char *_unreal_expand_string(const char *str, char *buf, size_t buflen, NameValuePrioList *nvp, int buildvarstring_options, Client *client);
 
 MOD_TEST()
 {
 	MARK_AS_OFFICIAL_MODULE(modinfo);
 	EfunctionAdd(modinfo->handle, EFUNC_REGISTER_USER, _register_user);
+	EfunctionAddString(modinfo->handle, EFUNC_UNREAL_EXPAND_STRING, _unreal_expand_string);
 	return MOD_SUCCESS;
 }
 
 MOD_INIT()
 {
+	MARK_AS_OFFICIAL_MODULE(modinfo);
+
 	CommandAdd(modinfo->handle, "NICK", cmd_nick, MAXPARA, CMD_USER|CMD_SERVER|CMD_UNREGISTERED);
 	CommandAdd(modinfo->handle, "UID", cmd_uid, MAXPARA, CMD_SERVER);
-	MARK_AS_OFFICIAL_MODULE(modinfo);
+
 	return MOD_SUCCESS;
 }
 
@@ -98,6 +100,31 @@ void set_user_modes_dont_spread(Client *client, const char *umode)
 	dontspread = 0;
 }
 
+/** Can user use this nick?
+ * This is only 1 part of many checks you would need to do.
+ * You should also run the nick through do_nick_name before this,
+ * and after this also check with find_qline() and find_client().
+ */
+int can_use_nick(Client *client, const char *nick)
+{
+	Hook *h;
+
+	for (h = Hooks[HOOKTYPE_CAN_USE_NICK]; h; h = h->next)
+	{
+		char *change_nick_error_from_hook = NULL;
+		int ret = (*(h->func.intfunc))(client, nick, &change_nick_error_from_hook);
+		if (ret == HOOK_DENY)
+		{
+			if (change_nick_error_from_hook)
+				sendnumeric(client, ERR_ERRONEUSNICKNAME, nick, change_nick_error_from_hook);
+			else
+				sendnumeric(client, ERR_ERRONEUSNICKNAME, nick, "Denied by hook");
+			return 0;
+		}
+	}
+	return 1;
+}
+
 /** Remote client (already fully registered) changing their nick */
 CMD_FUNC(cmd_nick_remote)
 {
@@ -270,10 +297,13 @@ CMD_FUNC(cmd_nick_local)
 	{
 		/* Local client changing nick: check spamfilter */
 		spamfilter_build_user_string(spamfilter_user, nick, client);
-		if (match_spamfilter(client, spamfilter_user, SPAMF_USER, "NICK", NULL, 0, NULL))
+		if (match_spamfilter(client, spamfilter_user, SPAMF_USER, "NICK", NULL, 0, clictx, NULL))
 			return;
 	}
 
+	if (!can_use_nick(client, nick))
+		return;
+
 	/* Check Q-lines / ban nick */
 	if (!IsULine(client) && (tklban = find_qline(client, nick, &ishold)))
 	{
@@ -654,7 +684,7 @@ nickkill2done:
 
 	serv = client;
 	client = make_client(serv->direction, serv);
-	strlcpy(client->id, parv[6], IDLEN);
+	strlcpy(client->id, parv[6], sizeof(client->id));
 	add_client_to_list(client);
 	add_to_id_hash_table(client->id, client);
 	client->lastnick = atol(parv[3]);
@@ -668,7 +698,22 @@ nickkill2done:
 	client->user->server = find_or_add(client->uplink->name);
 	strlcpy(client->user->realhost, hostname, sizeof(client->user->realhost));
 	if (ip)
-		safe_strdup(client->ip, ip);
+	{
+		if (!set_client_ip(client, ip))
+		{
+			/* This should not be possible as we validate the 'ip' with
+			 * the call to decode_ip() about 100 lines up.
+			 */
+			unreal_log(ULOG_ERROR, "nick", "REMOTE_CLIENT_IP_BUG", client,
+				   "[BUG] client $client has invalid ip $ip -- rejected",
+				   log_data_string("ip", ip));
+#ifdef DEBUGMODE
+			abort();
+#endif
+			/* This could leave a ghost / unsynched, but it should never happen... right? */
+			return;
+		}
+	}
 
 	if (*sstamp != '*')
 		strlcpy(client->user->account, sstamp, sizeof(client->user->account));
@@ -699,9 +744,6 @@ nickkill2done:
 	if (*virthost != '*')
 		safe_strdup(client->user->virthost, virthost);
 
-	/* Add to ipusers hash table (to track global maxperip) */
-	exceeds_maxperip(client, NULL);
-
 	build_umode_string(client, 0, SEND_UMODES|UMODE_SERVNOTICE, buf);
 
 	sendto_serv_butone_nickcmd(client->direction, recv_mtags, client, (*buf == '\0' ? "+" : buf));
@@ -1049,7 +1091,7 @@ int _register_user(Client *client)
 	find_shun(client);
 
 	spamfilter_build_user_string(spamfilter_user, client->name, client);
-	if (match_spamfilter(client, spamfilter_user, SPAMF_USER, NULL, NULL, 0, &savetkl))
+	if (match_spamfilter(client, spamfilter_user, SPAMF_USER, NULL, NULL, 0, NULL, &savetkl))
 	{
 		if (savetkl &&
 		    (has_actions_of_type(savetkl->ptr.spamfilter->action, BAN_ACT_VIRUSCHAN) ||
@@ -1223,53 +1265,6 @@ void nick_collision(Client *cptr, const char *newnick, const char *newid, Client
 	}
 }
 
-/** Returns 1 if allow::maxperip is exceeded by 'client' */
-int exceeds_maxperip(Client *client, ConfigItem_allow *aconf)
-{
-	Client *acptr;
-	IpUsersBucket *bucket;
-
-	if (!client->ip)
-		return 0; /* eg. services */
-
-	bucket = find_ipusers_bucket(client);
-	if (!bucket)
-	{
-		client->flags |= CLIENT_FLAG_IPUSERS_BUMPED;
-		bucket = add_ipusers_bucket(client);
-		bucket->global_clients = 1;
-		if (MyConnect(client))
-			bucket->local_clients = 1;
-		return 0;
-	}
-
-	/* Bump if we haven't done so yet
-	 * (Actually not sure if this can ever be false, but...
-	 *  who knows with some 3rd party or some future change)
-	 */
-	if (!(client->flags & CLIENT_FLAG_IPUSERS_BUMPED))
-	{
-		bucket->global_clients++;
-		if (MyConnect(client))
-			bucket->local_clients++;
-		client->flags |= CLIENT_FLAG_IPUSERS_BUMPED;
-	}
-
-	if (find_tkl_exception(TKL_MAXPERIP, client))
-		return 0; /* exempt */
-
-	if (aconf)
-	{
-		if ((bucket->local_clients > aconf->maxperip) ||
-		    (bucket->global_clients > aconf->global_maxperip))
-		{
-			return 1;
-		}
-	}
-
-	return 0;
-}
-
 /** Allow or reject the client based on allow { } blocks and all other restrictions.
  * @param client     Client to check (local)
  * @param username   Username, for some reason...
@@ -1283,6 +1278,7 @@ int AllowClient(Client *client)
 	char *hname;
 	static char uhost[HOSTLEN + USERLEN + 3];
 	static char fullname[HOSTLEN + 1];
+	Hook *h;
 
 	if (!IsSecure(client) && !IsLocalhost(client) && (iConf.plaintext_policy_user == POLICY_DENY))
 	{
@@ -1324,11 +1320,14 @@ int AllowClient(Client *client)
 		if (aconf->flags.useip)
 			set_sockhost(client, GetIP(client));
 
-		if (exceeds_maxperip(client, aconf))
+		for (h = Hooks[HOOKTYPE_ALLOW_CLIENT]; h; h = h->next)
 		{
-			/* Already got too many with that ip# */
-			exit_client(client, NULL, iConf.reject_message_too_many_connections);
-			return 0;
+			const char *reject_reason = (*(h->func.stringfunc))(client, aconf);
+			if (reject_reason)
+			{
+				exit_client(client, NULL, reject_reason);
+				return 0;
+			}
 		}
 
 		if (!((aconf->class->clients + 1) > aconf->class->maxclients))
@@ -1349,3 +1348,72 @@ int AllowClient(Client *client)
 	exit_client(client, NULL, iConf.reject_message_unauthorized);
 	return 0;
 }
+
+/** Expand 'str' which contains '$variables' into 'buf'.
+ * @param str		The string which may contain $variables
+ * @param buf		The buffer to store the result in
+ * @param buflen	The size of 'buf'
+ * @param nvp		A list with custom extra variables, or simply NULL.
+ *			Note that the list is freed at the end, including the custom settings.
+ * @param buildvarstring_options	One or more of BUILDVARSTRING_* or simply 0.
+ * @param client	The client to expand details from
+ * @returns A pointer to 'buf'
+ * @notes If you do provide a non-NULL 'nvp' then note that the entire nvp list is FREED before return
+ *        including your custom list!
+ */
+char *_unreal_expand_string(const char *str, char *buf, size_t buflen, NameValuePrioList *nvp, int buildvarstring_options, Client *client)
+{
+	const char *s;
+
+	if (client)
+	{
+		add_nvplist(&nvp, 0, "nick", client->name);
+		add_nvplist(&nvp, 0, "servername", client->uplink->name);
+		add_nvplist(&nvp, 0, "server", client->uplink->name); /* (old backwards compatible name) */
+		add_nvplist(&nvp, 0, "ip", GetIP(client));
+
+		if (client->user && *client->user->realhost)
+			add_nvplist(&nvp, 0, "hostname", client->user->realhost);
+		else if (client->local && *client->local->sockhost)
+			add_nvplist(&nvp, 0, "hostname", client->local->sockhost);
+		else
+			add_nvplist(&nvp, 0, "hostname", GetIP(client));
+
+		if (client->user)
+		{
+			add_nvplist(&nvp, 0, "username", client->user->username);
+			add_nvplist(&nvp, 0, "realname", client->info);
+			add_nvplist(&nvp, 0, "account", client->user->account);
+			s = get_operlogin(client);
+			if (s)
+				add_nvplist(&nvp, 0, "operlogin", s);
+			s = get_operclass(client);
+			if (s)
+				add_nvplist(&nvp, 0, "operclass", s);
+		}
+		if (client->ip)
+		{
+			GeoIPResult *geo = geoip_client(client);
+			if (geo)
+			{
+				char asn[32];
+				if (geo->country_code)
+					add_nvplist(&nvp, 0, "country_code", geo->country_code);
+				else
+					add_nvplist(&nvp, 0, "country_code", "XX");
+
+				/* Safe to set this unconditionally, will simply be 0
+				 * for things like localhost and such.
+				 */
+				snprintf(asn, sizeof(asn), "%d", geo->asn);
+				add_nvplist(&nvp, 0, "asn", asn);
+			} else {
+				add_nvplist(&nvp, 0, "country_code", "XX");
+				add_nvplist(&nvp, 0, "asn", "0");
+			}
+		}
+	}
+	buildvarstring_nvp(str, buf, buflen, nvp, buildvarstring_options);
+	safe_free_nvplist(nvp);
+	return buf;
+}
