@@ -271,6 +271,21 @@ void disable_ssl_protocols(SSL_CTX *ctx, TLSOptions *tlsoptions)
 #endif
 }
 
+/* Set TLS groups. This is an internal function that is used to
+ * prevent some ifdef mess. And yes, in OpenSSL 3.5.x and later
+ * they map to the same, but in OpenSSL 3.2.x-3.4.x they didn't.
+ */
+int unrealircd_set_tls_groups(SSL_CTX *ctx, const char *groups)
+{
+#if defined(HAS_SSL_CTX_SET1_GROUPS_LIST)
+	return SSL_CTX_set1_groups_list(ctx, groups);
+#elif defined(HAS_SSL_CTX_SET1_CURVES_LIST)
+	return SSL_CTX_set1_curves_list(ctx, groups);
+#else
+	return 0;
+#endif
+}
+
 /** Initialize TLS context
  * @param tlsoptions	The ::tls-options configuration
  * @param server	Set to 1 if we are initializing a server, 0 for client.
@@ -313,6 +328,28 @@ SSL_CTX *init_ctx(TLSOptions *tlsoptions, int server)
 #endif
 	SSL_CTX_set_options(ctx, SSL_OP_NO_TICKET);
 
+	/* Let's first check the simple case of file exist. That's because the
+	 * SSL_CTX_use_certificate_chain_file() later on works but produces like
+	 * four lines of output, which is a bit verbose for such a simple case.
+	 */
+	if (!file_exists(tlsoptions->certificate_file))
+	{
+		int saved_errno = errno;
+		unreal_log(ULOG_ERROR, "config", "TLS_LOAD_FAILED", NULL,
+		           "Could not open TLS certificate $filename: $system_error",
+		           log_data_string("filename", tlsoptions->certificate_file),
+		           log_data_string("system_error", strerror(saved_errno)));
+
+		if (str_ends_with_case_sensitive(tlsoptions->certificate_file, "tls/server.cert.pem"))
+		{
+			unreal_log(ULOG_ERROR, "config", "TLS_LOAD_FAILED_DEFAULT_CERT", NULL,
+			           "It seems the default certificate is missing. "
+			           "Run 'make pem && make install' in the UnrealIRCd source directory "
+			           "to generate a self-signed cert.");
+		}
+		goto fail;
+	}
+
 	if (SSL_CTX_use_certificate_chain_file(ctx, tlsoptions->certificate_file) <= 0)
 	{
 		unreal_log(ULOG_ERROR, "config", "TLS_LOAD_FAILED", NULL,
@@ -322,6 +359,17 @@ SSL_CTX *init_ctx(TLSOptions *tlsoptions, int server)
 		goto fail;
 	}
 
+	/* Let's first check the simple case of file exist - this time for key file. */
+	if (!file_exists(tlsoptions->key_file))
+	{
+		int saved_errno = errno;
+		unreal_log(ULOG_ERROR, "config", "TLS_LOAD_FAILED", NULL,
+		           "Could not open TLS key $filename: $system_error",
+		           log_data_string("filename", tlsoptions->key_file),
+		           log_data_string("system_error", strerror(saved_errno)));
+		goto fail;
+	}
+
 	if (SSL_CTX_use_PrivateKey_file(ctx, tlsoptions->key_file, SSL_FILETYPE_PEM) <= 0)
 	{
 		unreal_log(ULOG_ERROR, "config", "TLS_LOAD_FAILED", NULL,
@@ -412,31 +460,57 @@ SSL_CTX *init_ctx(TLSOptions *tlsoptions, int server)
 		 * do anything then, since auto ecdh is the default.
 		 */
 #endif
-		/* Let's see if we need to (and can) set specific curves */
-		if (tlsoptions->ecdh_curves)
+#if defined(HAS_SSL_CTX_SET1_CURVES_LIST) || defined(HAS_SSL_CTX_SET1_GROUPS_LIST)
+		/* Let's see if we need to set specific TLS groups */
+		if (tlsoptions->groups == NULL)
 		{
-#ifdef HAS_SSL_CTX_SET1_CURVES_LIST
-			if (!SSL_CTX_set1_curves_list(ctx, tlsoptions->ecdh_curves))
+			/* This means try the defaults.. */
+			if (!unrealircd_set_tls_groups(ctx, UNREALIRCD_DEFAULT_TLS_GROUPS_PRIMARY))
 			{
-				unreal_log(ULOG_ERROR, "config", "TLS_INVALID_ECDH_CURVES_LIST", NULL,
-					   "Failed to set ecdh-curves '$ecdh_curves_list'\n$tls_error.all\n"
-					   "HINT: o get a list of supported curves with the appropriate names, "
-					   "run 'openssl ecparam -list_curves' on the server. "
+				if (!unrealircd_set_tls_groups(ctx, UNREALIRCD_DEFAULT_TLS_GROUPS_SECONDARY))
+				{
+					if (!unrealircd_set_tls_groups(ctx, UNREALIRCD_DEFAULT_TLS_GROUPS_TERTIARY))
+					{
+						unreal_log(ULOG_ERROR, "config", "TLS_INVALID_TLS_GROUPS_LIST", NULL,
+							   "Failed to set groups / ecdh-curves to either "
+							   "'$tls_groups_primary', '$tls_groups_secondary' or '$tls_groups_tertiary'.\n"
+							   "$tls_error.all\n"
+							   "It's strange that none of the three worked. "
+							   "Please report at https://bugs.unrealircd.org/ !",
+							   log_data_string("tls_groups_primary", UNREALIRCD_DEFAULT_TLS_GROUPS_PRIMARY),
+							   log_data_string("tls_groups_secondary", UNREALIRCD_DEFAULT_TLS_GROUPS_SECONDARY),
+							   log_data_string("tls_groups_tertiary", UNREALIRCD_DEFAULT_TLS_GROUPS_TERTIARY),
+							   log_data_tls_error());
+						goto fail;
+					}
+				}
+			}
+		} else
+		{
+			/* User-configured TLS groups */
+			if (!unrealircd_set_tls_groups(ctx, tlsoptions->groups))
+			{
+				unreal_log(ULOG_ERROR, "config", "TLS_INVALID_TLS_GROUPS_LIST", NULL,
+					   "Failed to set groups / ecdh-curves '$tls_groups'\n$tls_error.all\n"
+					   "HINT: To get a list of supported names, run 'openssl ecparam -list_curves' on the server. "
 					   "Separate multiple curves by colon, for example: "
-					   "ecdh-curves \"secp521r1:secp384r1\".",
-					   log_data_string("ecdh_curves_list", tlsoptions->ecdh_curves),
+					   "groups \"secp521r1:secp384r1\".",
+					   log_data_string("tls_groups", tlsoptions->groups),
 					   log_data_tls_error());
 				goto fail;
 			}
+		}
 #else
+		if (tlsoptions->groups)
+		{
 			/* We try to avoid this in the config code, but better have
 			 * it here too than be sorry if someone screws up:
 			 */
-			unreal_log(ULOG_ERROR, "config", "BUG_ECDH_CURVES", NULL,
+			unreal_log(ULOG_ERROR, "config", "BUG_TLS_GROUPS", NULL,
 			           "ecdh-curves specified but not supported by library -- BAD!");
 			goto fail;
-#endif
 		}
+#endif
 		/* We really want the ECDHE/ECDHE to be generated per-session.
 		 * Added in 2015 for safety. Seems OpenSSL was smart enough
 		 * to make this the default in 2016 after a security advisory.
@@ -461,6 +535,107 @@ MODVAR EVP_MD *sha1_function; /**< SHA1 function for EVP_DigestInit_ex() call */
 MODVAR EVP_MD *md5_function; /**< MD5 function for EVP_DigestInit_ex() call */
 #endif
 
+/** Check if a SSL_CTX is trusted by the associated trust store
+ * This only works with OpenSSL 1.1.0 or later (it could have
+ * worked with lower versions if accessing structs directly but
+ * i didn't bother).
+ */
+int is_trusted_cert(SSL_CTX *ctx)
+{
+	int ok = 0;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	X509 *cert = NULL;
+	STACK_OF(X509) *chain = NULL;
+	X509_STORE *store = NULL;
+	X509_STORE_CTX *vctx = NULL;
+
+	cert = SSL_CTX_get0_certificate(ctx);
+	SSL_CTX_get0_chain_certs(ctx, &chain);
+	if (!cert)
+		return 0;
+
+	store = SSL_CTX_get_cert_store(ctx);
+	if (!store)
+		return 0;
+
+	vctx = X509_STORE_CTX_new();
+	if (!vctx)
+		goto is_trusted_cert_done;
+
+	if (!X509_STORE_CTX_init(vctx, store, cert, chain))
+		goto is_trusted_cert_done;
+
+	if (X509_verify_cert(vctx) == 1)
+		ok = 1;
+
+is_trusted_cert_done:
+	X509_STORE_CTX_free(vctx);
+#endif
+	return ok;
+}
+
+/** Tests if any of the loaded certs is trusted by a CA.
+ * Note that we don't check if it is the one exposed to
+ * clients or if it is valid for the hostname or is expired.
+ */
+int has_any_trusted_cert(void)
+{
+	ConfigItem_listen *listen;
+	ConfigItem_sni *sni;
+
+	if (ctx_server && is_trusted_cert(ctx_server))
+		return 1;
+	if (ctx_client && is_trusted_cert(ctx_client))
+		return 1;
+
+	for (listen = conf_listen; listen; listen = listen->next)
+		if (listen->ssl_ctx && is_trusted_cert(listen->ssl_ctx))
+			return 1;
+
+	for (sni = conf_sni; sni; sni = sni->next)
+		if (sni->ssl_ctx && is_trusted_cert(sni->ssl_ctx))
+			return 1;
+
+	return 0; /* Zero trusted certs found */
+}
+
+/** Checks if a ctx is valid for specified hostname.
+ * In case of doubt this function will return 1 (!),
+ * so use it in diagnostics only! Eg config tests.
+ */
+int check_ctx_valid_for_hostname(SSL_CTX *ctx, const char *hostname)
+{
+#ifdef HAS_X509_check_host
+	X509 *cert = SSL_CTX_get0_certificate(ctx);
+	if (!cert)
+		return 0;
+	if (X509_check_host(cert, hostname, 0, 0, NULL) == 0)
+		return 0;
+#endif
+	return 1;
+}
+
+int has_any_trusted_cert_with_correct_hostname(void)
+{
+	ConfigItem_listen *listen;
+	ConfigItem_sni *sni;
+
+	if (ctx_server && is_trusted_cert(ctx_server) && check_ctx_valid_for_hostname(ctx_server, me.name))
+		return 1;
+	if (ctx_client && is_trusted_cert(ctx_client) && check_ctx_valid_for_hostname(ctx_client, me.name))
+		return 1;
+
+	for (listen = conf_listen; listen; listen = listen->next)
+		if (listen->ssl_ctx && is_trusted_cert(listen->ssl_ctx) && check_ctx_valid_for_hostname(listen->ssl_ctx, me.name))
+			return 1;
+
+	for (sni = conf_sni; sni; sni = sni->next)
+		if (sni->ssl_ctx && is_trusted_cert(sni->ssl_ctx) && check_ctx_valid_for_hostname(sni->ssl_ctx, me.name))
+			return 1;
+
+	return 0; /* Zero trusted certs found */
+}
+
 /** Early initalization of TLS subsystem - called on startup */
 int early_init_tls(void)
 {
@@ -486,11 +661,6 @@ int early_init_tls(void)
 	}
 
 	md5_function = EVP_MD_fetch(NULL, "MD5", NULL);
-	if (!md5_function)
-	{
-		fprintf(stderr, "Could not find MD5 algorithm in TLS library\n");
-		exit(6);
-	}
 #endif
 	return 1;
 }
@@ -609,7 +779,8 @@ void SSL_set_nonblocking(SSL *s)
 const char *tls_get_cipher(Client *client)
 {
 	static char buf[256];
-	const char *cached;
+	const char *cached, *s;
+	int group;
 
 	cached = moddata_client_get(client, "tls_cipher");
 	if (cached)
@@ -620,7 +791,20 @@ const char *tls_get_cipher(Client *client)
 
 	buf[0] = '\0';
 	strlcpy(buf, SSL_get_version(client->local->ssl), sizeof(buf));
-	strlcat(buf, "-", sizeof(buf));
+#ifdef HAS_SSL_GET_NEGOTIATED_GROUP
+	group = SSL_get_negotiated_group(client->local->ssl);
+	s = SSL_group_to_name(client->local->ssl, group);
+	if (s)
+	{
+		/* "x25519" -> "X25519" */
+		char gbuf[64];
+		strlcat(buf, "/", sizeof(buf));
+		*gbuf = '\0';
+		strtoupper_safe(gbuf, s, sizeof(gbuf));
+		strlcat(buf, gbuf, sizeof(buf));
+	}
+#endif
+	strlcat(buf, "/", sizeof(buf));
 	strlcat(buf, SSL_get_cipher(client->local->ssl), sizeof(buf));
 
 	return buf;
@@ -871,6 +1055,10 @@ static int fatal_tls_error(int ssl_error, int where, int my_errno, Client *clien
 	char buf[512];
 	const char *one, *two;
 
+	/* deregister I/O notification since we don't care anymore. the actual closing of socket will happen later. */
+	if (client->local->fd >= 0)
+		fd_unnotify(client->local->fd);
+
 	if (IsDeadSocket(client))
 		return -1;
 
@@ -917,10 +1105,12 @@ static int fatal_tls_error(int ssl_error, int where, int my_errno, Client *clien
 
 	SetDeadSocket(client);
 	unreal_log(ULOG_DEBUG, "tls", "DEBUG_TLS_FATAL_ERROR", client,
-		   "Exiting TLS client $client.details: $tls_function: $tls_error_string: $tls_additional_info",
+		   "Exiting TLS client $client.details [port $port]: $tls_function: $tls_error_string: $tls_additional_info",
 		   log_data_string("tls_function", ssl_func),
 		   log_data_string("tls_error_string", ssl_errstr),
-		   log_data_string("tls_additional_info", additional_info));
+		   log_data_string("tls_additional_info", additional_info),
+		   log_data_integer("port", client->local->listener ? client->local->listener->port : 0));
+
 
 	if (where == FUNC_TLS_CONNECT)
 	{
@@ -952,10 +1142,6 @@ static int fatal_tls_error(int ssl_error, int where, int my_errno, Client *clien
 		safe_strdup(client->local->error_str, ssl_errstr);
 	}
 
-	/* deregister I/O notification since we don't care anymore. the actual closing of socket will happen later. */
-	if (client->local->fd >= 0)
-		fd_unnotify(client->local->fd);
-
 	return -1;
 }
 
@@ -1043,19 +1229,23 @@ int verify_certificate(SSL *ssl, const char *hostname, char **errstr)
 	if (!ssl)
 	{
 		strlcpy(buf, "Not using TLS", sizeof(buf));
-		if (errstr)
-			*errstr = buf;
-		return 0; /* Cannot verify a non-TLS connection */
+		goto verify_certificate_failed; /* Cannot verify a non-TLS connection */
 	}
 
-	if (SSL_get_verify_result(ssl) != X509_V_OK)
+	n = SSL_get_verify_result(ssl);
+
+	if (n == X509_V_ERR_CERT_HAS_EXPIRED)
 	{
-		// FIXME: there are actually about 25+ different possible errors,
-		// this is only the most common one:
+		strlcpy(buf, "Certificate is expired", sizeof(buf));
+		goto verify_certificate_failed;
+	}
+
+	if (n != X509_V_OK)
+	{
+		// There are actually about 25+ different possible errors,
+		// let's pretend it is the most common one:
 		strlcpy(buf, "Certificate is not issued by a trusted Certificate Authority", sizeof(buf));
-		if (errstr)
-			*errstr = buf;
-		return 0; /* Certificate verify failed */
+		goto verify_certificate_failed;
 	}
 
 	/* Now verify if the name of the certificate matches hostname */
@@ -1093,6 +1283,7 @@ int verify_certificate(SSL *ssl, const char *hostname, char **errstr)
 	/* Certificate is verified but is issued for a different hostname */
 	snprintf(buf, sizeof(buf), "Certificate '%s' is not valid for hostname '%s'",
 		certificate_name(ssl), hostname);
+verify_certificate_failed:
 	if (errstr)
 		*errstr = buf;
 	return 0;
@@ -1418,7 +1609,7 @@ void check_certificate_expiry_tlsoptions_and_warn(TLSOptions *tlsoptions)
 
 	if (check_certificate_expiry_ctx(ctx, &errstr))
 	{
-		unreal_log(ULOG_ERROR, "tls", "TLS_CERT_EXPIRING", NULL,
+		unreal_log(ULOG_WARNING, "tls", "TLS_CERT_EXPIRING", NULL,
 		           "Warning: TLS certificate '$filename': $error_string",
 		           log_data_string("filename", tlsoptions->certificate_file),
 		           log_data_string("error_string", errstr));
@@ -1428,6 +1619,9 @@ void check_certificate_expiry_tlsoptions_and_warn(TLSOptions *tlsoptions)
 
 EVENT(tls_check_expiry)
 {
+	if (iConf.tls_options->certificate_expiry_notification == 0)
+		return;
+
 	ConfigItem_listen *listen;
 	ConfigItem_sni *sni;
 	ConfigItem_link *link;
